import psycopg2

from client import client

START_TABLE = 'ir_values'
START_TABLE = 'ir_ui_menu'
START_TABLE = 'ir_act_window'
IDS_TO_DROP = [495, 496, 497, 500, 501, 502, 503, 665, 666, 670, 671, 672, 674, 675, 676]

def pad(text, width):
    if width >= 0:
        trimmed = text[:width]
        return trimmed + ' ' * (width - len(trimmed))
    trimmed = text[:-width]
    return ' ' * (-width - len(trimmed)) + trimmed

def print_descendants(indentation, table_name, ids, cr, tables):
    ids_query = ','.join(map(str, ids))
    tables.append((table_name, ids_query, 'id'))

    main_query = """
SELECT  'SELECT ''' || tc.table_name || 
        ''' as tabname, ''' || kcu.column_name || 
        ''' as colname, ' || kcu.column_name || 
        ' as id, count(*) FROM ' || tc.table_name || 
        ' as cnt WHERE ' || kcu.column_name || ' IN (%s) ' ||
        ' GROUP BY ' || kcu.column_name AS command
FROM    information_schema.table_constraints AS tc 
JOIN    information_schema.key_column_usage AS kcu 
ON      tc.constraint_name = kcu.constraint_name
JOIN    information_schema.constraint_column_usage AS ccu
ON      ccu.constraint_name = tc.constraint_name
WHERE   constraint_type = 'FOREIGN KEY'
AND     ccu.table_name = '%s';
""" % (ids_query, table_name)
    cr.execute(main_query)
    rows = cr.fetchall()
    
    if not rows:
        return
    detail_query = ' UNION ALL '.join([row[0] for row in rows])
    cr.execute(detail_query)
    rows = cr.fetchall()
    
    for child_name, column_name, parent_id, count in rows:
        print (pad(indentation + child_name, 50) +
               pad(column_name, 20) +
               pad(str(parent_id), 30) +
               pad(str(count), -10))
        if child_name.endswith('_rel'):
            tables.append((child_name, ids_query, column_name))
        else:
            child_query = """
SELECT  id
FROM    %s
WHERE   %s in (%s);
""" % (child_name, column_name, ids_query)
            cr.execute(child_query)
            child_rows = cr.fetchall()
            child_ids = [child_row[0] for child_row in child_rows]
            print_descendants(indentation + '    ', 
                              child_name, 
                              child_ids, 
                              cr,
                              tables)

c = client()
#connection will use details from ~/.pgpass file.
conn = psycopg2.connect('dbname=%s' % c.dbname)
cr = conn.cursor()
tables = []

print (pad('table name', 50) + 
       pad('column name', 20) +
       pad('id', 30) +
       pad('count', -10))
print_descendants('', START_TABLE, IDS_TO_DROP, cr, tables)

choice = raw_input("Delete all records? (y/n)")
if choice.lower() != 'y':
    print 'Not deleted.'
else:
    while (tables):
        table_name, ids_query, column_name = tables.pop()
        cr.execute('DELETE FROM %s WHERE %s IN (%s)' % (table_name, 
                                                        column_name, 
                                                        ids_query))
        cr.execute('COMMIT;')
    print 'Deleted all records.'